@@ -23,7 +23,6 @@
 #define APPLETINTERFACE_H
 
 #include <QQuickItem>
-#include <QScriptValue>
 #include <QQuickView>
 
 #include <Plasma/Applet>
@@ -90,6 +89,25 @@ class AppletInterface : public PlasmaQuick::AppletQuickItem
      */
     Q_PROPERTY(QString toolTipSubText READ toolTipSubText WRITE setToolTipSubText NOTIFY toolTipSubTextChanged)
 
+    /**
+     * how to handle the text format of the tooltip subtext:
+     * * Text.AutoText (default)
+     * * Text.PlainText
+     * * Text.StyledText
+     * * Text.RichText
+     * Note: in the default implementation the main text is always plain text
+     */
+    Q_PROPERTY(int toolTipTextFormat READ toolTipTextFormat WRITE setToolTipTextFormat NOTIFY toolTipTextFormatChanged)
+
+    /**
+     * This allows to set custom full QML items as the tooltip.
+     * It will ignore all texts set by setToolTipMainText or setToolTipSubText
+     *
+     * @since: 5.19
+     */
+    Q_PROPERTY(QQuickItem *toolTipItem READ toolTipItem WRITE setToolTipItem NOTIFY toolTipItemChanged)
+
+
     /**
      * Icon to represent the plasmoid
      */
@@ -131,9 +149,21 @@ class AppletInterface : public PlasmaQuick::AppletQuickItem
     Q_PROPERTY(Plasma::Types::BackgroundHints backgroundHints WRITE setBackgroundHints READ backgroundHints NOTIFY backgroundHintsChanged)
 
     /**
-     * The immutability state of the Corona. the plasmoid implementation should avoid allowing "dangerous" modifications from the user when in an immutable mode
+     * Whether the Corona is immutable. The plasmoid implementation should avoid allowing "dangerous" modifications from the user when in an immutable mode
+     *
+     * This is true when immutability is not Mutable
      */
-    Q_PROPERTY(bool immutable READ immutable NOTIFY immutableChanged)
+    Q_PROPERTY(bool immutable READ immutable NOTIFY immutabilityChanged)
+
+    /**
+     * The immutability of the Corona.
+     *
+     * Use this if you need more granular control than just using the immutable property
+     *
+     * @see immutable
+     * @since 5.23
+     */
+    Q_PROPERTY(Plasma::Types::ImmutabilityType immutability READ immutability NOTIFY immutabilityChanged)
 
     /**
      * True when the user is configuring, for instance when the configuration dialog is open.
@@ -158,7 +188,7 @@ class AppletInterface : public PlasmaQuick::AppletQuickItem
     // TODO: This was moved up from ContainmentInterface because it is required by the
     // Task Manager applet (for "Show only tasks from this screen") and no Qt API exposes
     // screen numbering. An alternate solution that doesn't extend the applet interface
-    // would be preferrable if found.
+    // would be preferable if found.
     Q_PROPERTY(int screen READ screen NOTIFY screenChanged)
 
     /**
@@ -182,9 +212,23 @@ class AppletInterface : public PlasmaQuick::AppletQuickItem
      */
     Q_PROPERTY(QKeySequence globalShortcut READ globalShortcut WRITE setGlobalShortcut NOTIFY globalShortcutChanged)
 
+    /**
+     * An interface to the native C++ plasmoid, if implemented
+     */
+    Q_PROPERTY(QObject *nativeInterface READ nativeInterface CONSTANT)
+
+    /**
+     * If true the applet requires manual configuration from the user
+     */
+    Q_PROPERTY(bool configurationRequired READ configurationRequired WRITE setConfigurationRequiredProperty NOTIFY configurationRequiredChanged)
+
+    /**
+     * Reason why the manual user configuration is required
+     */
+    Q_PROPERTY(QString configurationRequiredReason READ configurationRequiredReason WRITE setConfigurationRequiredReason NOTIFY configurationRequiredReasonChanged)
+
 public:
     AppletInterface(DeclarativeAppletScript *script, const QVariantList &args = QVariantList(), QQuickItem *parent = 0);
-    AppletInterface(Plasma::Applet *applet, const QVariantList &args = QVariantList(), QQuickItem *parent = 0);
     ~AppletInterface();
 
 //API not intended for the QML part
@@ -218,6 +262,8 @@ public:
 
     Q_INVOKABLE void removeAction(const QString &name);
 
+    Q_INVOKABLE void clearActions();
+
     Q_INVOKABLE QAction *action(QString name) const;
 
     /**
@@ -238,9 +284,17 @@ public:
     Q_INVOKABLE QString file(const QString &fileType, const QString &filePath);
 
     /**
-     * @returns A path where is safe to write on disk downloaded files
+     * @returns A path where it is safe to write on disk downloaded files.
+     * @since 5.23
      */
-    Q_INVOKABLE QString downloadPath(const QString &file);
+    Q_INVOKABLE QString downloadPath() const;
+
+    /**
+     * @returns A path where it is safe to write on disk downloaded files.
+     * @param file that name of the file to download (unused).
+     * @deprecated Use downloadPath() instead.
+     */
+    Q_INVOKABLE PLASMA_DEPRECATED QString downloadPath(const QString &file);
 
     /**
      * @returns The list of files that have been downloaded
@@ -267,6 +321,12 @@ public:
     QString toolTipSubText() const;
     void setToolTipSubText(const QString &text);
 
+    int toolTipTextFormat() const;
+    void setToolTipTextFormat(int format);
+
+    QQuickItem *toolTipItem() const;
+    void setToolTipItem(QQuickItem *toolTipItem);
+
     uint id() const;
 
     Plasma::Types::FormFactor formFactor() const;
@@ -296,6 +356,7 @@ public:
     QRect screenGeometry() const;
 
     bool immutable() const;
+    Plasma::Types::ImmutabilityType immutability() const;
     bool userConfiguring() const;
     int apiVersion() const;
 
@@ -305,6 +366,16 @@ public:
     QKeySequence globalShortcut() const;
     void setGlobalShortcut(const QKeySequence &keySequence);
 
+    QObject *nativeInterface();
+
+    //NOTE: setConfigurationRequiredProperty because ambiguous with the
+    // setConfigurationRequired invokable
+    bool configurationRequired() const;
+    void setConfigurationRequiredProperty(bool required);
+
+    QString configurationRequiredReason() const;
+    void setConfigurationRequiredReason(const QString &reason);
+
 Q_SIGNALS:
     /**
      * somebody else, usually the containment sent some data to the applet
@@ -332,10 +403,12 @@ Q_SIGNALS:
     void titleChanged();
     void toolTipMainTextChanged();
     void toolTipSubTextChanged();
+    void toolTipTextFormatChanged();
+    void toolTipItemChanged();
     void formFactorChanged();
     void locationChanged();
     void contextChanged();
-    void immutableChanged();
+    void immutabilityChanged();
     void statusChanged();
     void backgroundHintsChanged();
     void busyChanged();
@@ -346,12 +419,15 @@ Q_SIGNALS:
 
     void userConfiguringChanged();
     void globalShortcutChanged();
+    void configurationRequiredChanged();
+    void configurationRequiredReasonChanged();
 
 protected Q_SLOTS:
-    virtual void init();
+    void init() Q_DECL_OVERRIDE;
 
 protected:
-    bool eventFilter(QObject *watched, QEvent *event);
+    bool event(QEvent *event) Q_DECL_OVERRIDE;
+    bool eventFilter(QObject *watched, QEvent *event) Q_DECL_OVERRIDE;
 
 private Q_SLOTS:
     void destroyedChanged(bool destroyed);
@@ -368,10 +444,14 @@ private:
 
     QString m_toolTipMainText;
     QString m_toolTipSubText;
+    int m_toolTipTextFormat;
+    QPointer<QQuickItem> m_toolTipItem;
     QVariantList m_args;
     Plasma::Types::BackgroundHints m_backgroundHints;
-    bool m_busy : 1;
     bool m_hideOnDeactivate : 1;
+    //this is used to build an emacs style shortcut
+    int m_oldKeyboardShortcut;
+    QObject *m_dummyNativeInterface;
 
     friend class ContainmentInterface;
     //This is used by ContainmentInterface
