@@ -21,10 +21,9 @@
 
 #include "containmentinterface.h"
 #include <kdeclarative/configpropertymap.h>
-#include <kdeclarative/qmlobject.h>
+#include <kdeclarative/qmlobjectsharedengine.h>
 
 #include <kactioncollection.h>
-#include <kservicetypetrader.h>
 #include <kdesktopfile.h>
 #include <KConfigLoader>
 
@@ -35,6 +34,7 @@
 #include <QSignalMapper>
 
 #include <Plasma/PluginLoader>
+#include <kpackage/packageloader.h>
 
 QHash<QObject *, WallpaperInterface *> WallpaperInterface::s_rootObjects = QHash<QObject *, WallpaperInterface *>();
 
@@ -48,6 +48,11 @@ WallpaperInterface::WallpaperInterface(ContainmentInterface *parent)
 {
     m_actions = new KActionCollection(this);
 
+    //resize at the beginning to avoid as much resize events as possible
+    if (parent) {
+        setSize(QSizeF(parent->width(), parent->height()));
+    }
+
     if (!m_containmentInterface->containment()->wallpaper().isEmpty()) {
         syncWallpaperPackage();
     }
@@ -65,14 +70,14 @@ WallpaperInterface::~WallpaperInterface()
 
 KPluginInfo::List WallpaperInterface::listWallpaperInfoForMimetype(const QString &mimetype, const QString &formFactor)
 {
-    QString constraint = QString("'%1' in [X-Plasma-DropMimeTypes]").arg(mimetype);
-    if (!formFactor.isEmpty()) {
-        constraint.append("[X-Plasma-FormFactors] ~~ '").append(formFactor).append("'");
-    }
-
-    KService::List offers = KServiceTypeTrader::self()->query("Plasma/Wallpaper", constraint);
-    qDebug() << offers.count() << constraint;
-    return KPluginInfo::fromServices(offers);
+    auto filter = [&mimetype, &formFactor](const KPluginMetaData &md) -> bool
+    {
+        if (!formFactor.isEmpty() && !md.value(QStringLiteral("X-Plasma-FormFactors")).contains(formFactor)) {
+            return false;
+        }
+        return md.value(QStringLiteral("X-Plasma-DropMimeTypes")).contains(mimetype);
+    };
+    return KPluginInfo::fromMetaData(KPackage::PackageLoader::self()->findPackages(QStringLiteral("Plasma/Wallpaper"), QString(), filter).toVector());
 }
 
 Plasma::Package WallpaperInterface::package() const
@@ -94,16 +99,17 @@ KConfigLoader *WallpaperInterface::configScheme()
 {
     if (!m_configLoader) {
         //FIXME: do we need "mainconfigxml" in wallpaper packagestructures?
-        const QString xmlPath = m_pkg.filePath("config", "main.xml");
+        const QString xmlPath = m_pkg.filePath("config", QStringLiteral("main.xml"));
 
         KConfigGroup cfg = m_containmentInterface->containment()->config();
         cfg = KConfigGroup(&cfg, "Wallpaper");
+        cfg = KConfigGroup(&cfg, m_wallpaperPlugin);
 
         if (xmlPath.isEmpty()) {
-            m_configLoader = new KConfigLoader(cfg, 0);
+            m_configLoader = new KConfigLoader(cfg, 0, this);
         } else {
             QFile file(xmlPath);
-            m_configLoader = new KConfigLoader(cfg, &file);
+            m_configLoader = new KConfigLoader(cfg, &file, this);
         }
     }
 
@@ -120,21 +126,22 @@ void WallpaperInterface::syncWallpaperPackage()
     m_wallpaperPlugin = m_containmentInterface->containment()->wallpaper();
 
     if (!m_qmlObject) {
-        m_qmlObject = new KDeclarative::QmlObject(this);
+        m_qmlObject = new KDeclarative::QmlObjectSharedEngine(this);
         s_rootObjects[m_qmlObject->engine()] = this;
         m_qmlObject->setInitializationDelayed(true);
+        connect(m_qmlObject, &KDeclarative::QmlObject::finished, this, &WallpaperInterface::loadFinished);
     }
 
     m_actions->clear();
-    m_pkg = Plasma::PluginLoader::self()->loadPackage("Plasma/Wallpaper");
+    m_pkg = Plasma::PluginLoader::self()->loadPackage(QStringLiteral("Plasma/Wallpaper"));
+    m_pkg.setPath(m_wallpaperPlugin);
     if (!m_pkg.isValid()) {
         qWarning() << "Error loading the wallpaper, no valid package loaded";
         return;
     }
-    m_pkg.setPath(m_wallpaperPlugin);
 
-    m_configLoader->deleteLater();
-    m_configuration->deleteLater();
+    if (m_configLoader) m_configLoader->deleteLater();
+    if (m_configuration) m_configuration->deleteLater();
     m_configLoader = 0;
     m_configuration = 0;
     if (configScheme()) {
@@ -142,9 +149,24 @@ void WallpaperInterface::syncWallpaperPackage()
     }
 
     m_qmlObject->setSource(QUrl::fromLocalFile(m_pkg.filePath("mainscript")));
-    m_qmlObject->engine()->rootContext()->setContextProperty("wallpaper", this);
-    m_qmlObject->completeInitialization();
+    m_qmlObject->rootContext()->setContextProperty(QStringLiteral("wallpaper"), this);
 
+    const QString rootPath = m_pkg.metadata().property(QStringLiteral("X-Plasma-RootPath")).toString();
+    if (!rootPath.isEmpty()) {
+        m_qmlObject->setTranslationDomain(QLatin1String("plasma_wallpaper_") + rootPath);
+    } else {
+        m_qmlObject->setTranslationDomain(QLatin1String("plasma_wallpaper_") + m_pkg.metadata().pluginName());
+    }
+
+    //initialize with our size to avoid as much resize events as possible
+    QVariantHash props;
+    props[QStringLiteral("width")] = width();
+    props[QStringLiteral("height")] = height();
+    m_qmlObject->completeInitialization(props);
+}
+
+void WallpaperInterface::loadFinished()
+{
     if (m_qmlObject->mainComponent() &&
             m_qmlObject->rootObject() &&
             !m_qmlObject->mainComponent()->isError()) {
@@ -152,8 +174,8 @@ void WallpaperInterface::syncWallpaperPackage()
         m_qmlObject->rootObject()->setProperty("parent", QVariant::fromValue(this));
 
         //set anchors
-        QQmlExpression expr(m_qmlObject->engine()->rootContext(), m_qmlObject->rootObject(), "parent");
-        QQmlProperty prop(m_qmlObject->rootObject(), "anchors.fill");
+        QQmlExpression expr(m_qmlObject->engine()->rootContext(), m_qmlObject->rootObject(), QStringLiteral("parent"));
+        QQmlProperty prop(m_qmlObject->rootObject(), QStringLiteral("anchors.fill"));
         prop.write(expr.evaluate());
 
     } else if (m_qmlObject->mainComponent()) {
@@ -185,7 +207,7 @@ bool WallpaperInterface::supportsMimetype(const QString &mimetype) const
 void WallpaperInterface::setUrl(const QUrl &url)
 {
     if (m_qmlObject->rootObject()) {
-        QMetaObject::invokeMethod(m_qmlObject->rootObject(), QString("setUrl").toLatin1(), Qt::DirectConnection, Q_ARG(QVariant, QVariant::fromValue(url)));
+        QMetaObject::invokeMethod(m_qmlObject->rootObject(), QStringLiteral("setUrl").toLatin1(), Qt::DirectConnection, Q_ARG(QVariant, QVariant::fromValue(url)));
     }
 }
 
